/***************************************************************************
                          squeezelabel.cpp  -  description
                             -------------------
    begin                : sun aug 10 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    Copyright: See COPYING file that comes with this project
***************************************************************************/

#include "squeezelabel.h"

#include <qregexp.h>
#include <qtooltip.h>
#include <qstringlist.h>


SqueezeLabel::SqueezeLabel( QWidget * parent, const char * name, WFlags f )
	: QLabel( parent, name, f ), mShowTip( TRUE )
{
	setScaledContents( TRUE );
}


void SqueezeLabel::setShowToolTip( bool show )
{
	mShowTip = show;
}


void SqueezeLabel::setText( const QString & text )
{
	if ( ! hasScaledContents() )
		QLabel::setText( text );
	else {
		mFullText = text;
		setSqueezingText( width() );
	}
}


void SqueezeLabel::setSqueezingText( uint maxWidth )
{
	if ( ! hasScaledContents() )
		return;

	QString fullText = mFullText;

	if ( mFullText.contains('\n') ) {
		fullText = "";
		QStringList textLines = QStringList::split( '\n', mFullText, TRUE );
		for ( uint i=0; i<textLines.count(); i++)
			fullText += squeezeLine( textLines[i], maxWidth );
	}
	else
		fullText = squeezeLine( mFullText, maxWidth );

	if ( fullText.at(fullText.length()-1) == '\n' )
		fullText.remove( fullText.length()-1, 1 ); // remove last "\n"
// 	fullText.replace( QRegExp("\\n\\n"), "\n\n\n\n\n\n" ); // need to fix bug in Qt-3.2.0
	QLabel::setText( fullText );

	int p;
	if ( (p=fullText.find("...")) != -1 && fullText != "..." ) {
		if ( mShowTip ) {
			QToolTip::remove( this ); // remove previous tip
			QToolTip::add( this, mFullText );
		}
	}
	else
		if ( mShowTip )
			QToolTip::remove( this );
}


QString SqueezeLabel::squeezeLine( const QString & line, uint maxWidth )
{
	if ( line.isEmpty() )
		return QString("\n");

	QString newLine = line;
	// replace each occurence of tab char. to 8 space char.
	newLine.replace( QRegExp("\\t"), "        " );

	// TODO
	// need to add rich text support !

	QString leftHalfOfText, rightHalfOfText;
	uint textWidthInPixels = fontMetrics().width( newLine ), cutPoint = 0;

	if ( maxWidth < textWidthInPixels )  {
		cutPoint = ((textWidthInPixels - fontMetrics().width("...") - maxWidth+10) / font().pointSize()) + 1;
		leftHalfOfText  = newLine.left( (newLine.length()/2)-cutPoint );
		rightHalfOfText = newLine.right( (newLine.length()/2)-cutPoint );

		newLine = leftHalfOfText + "..." + rightHalfOfText;
	}

	return newLine+"\n";
}


void SqueezeLabel::resizeEvent( QResizeEvent * )
{
	setSqueezingText( width() );
}
