/***************************************************************************
                          panel.h  -  description
                             -------------------
    begin                : sat oct 26 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _PANEL_H_
#define _PANEL_H_

#include "filespanel.h"
#include "fileview.h"
#include "enums.h"

#include <qwidget.h>
#include <qstringlist.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa bazowa dla obiektu panela.
 Tworzony jest tutaj i usuwany podany panel (listy plikw lub podgld pliku).
 Klasa zawiera wiele metod poredniczcych pomidzy obiektem klasy nadrzdnej
 tj. rodzica dla biecej (np. QtCmd) oraz podrzdnej - potomka (np. FilesPanel)
 Metody wspomagaj komunikacj pomidzy panelami tych samych typw.
*/
class Panel : public QWidget
{
public:
	/** Konstruktor klasy.\n
	 @param w - szeroko panelu,
	 @param h - wysoko panelu,
	 @param parent - wkanik na rodzica panelu (obiekt klasy QSplitter),
	 @param preParent - wkanik na rodzica dla rodzica panelu (obiekt klasy QtCmd)
	 @param name - nazwa dla obiektu.\n
	 Inicjalizowane s tutaj skadowe, oraz ustawiany rozmiar panela.
	 */
	Panel( uint w, uint h, QWidget *parent, QWidget *preParent=0, const char * name=0 );

	/** Destruktor klasy.\n
	 Zapisywane s tutaj do pliku konfiguracyjnego ustawienia takie jak:
	 jeli biecym panelem jest podgld pliku - to jego absolutna nazwa,
	 jeli biecym panelem jest lista plikw - to jego szeroko oraz rodzaj
	 widoku (drzewo/lista). Usuwany jest take obiekt panela.
	 */
	~Panel();

	/** Podaje typ biecego panela.\n
	 @return typ panela, patrz @see KindOfPanel.
	 */
	KindOfPanel kindOfPanel() const { return mKindOfPanel; }

	/** Zwraca rodzaj widoku w biecym panelu.\n
	 @return rodzaj widoku, patrz: @see KindOfListView i @see View::KindOfView.
	 */
	int kindOfView() const;

	/** Zwraca informacj o tym czy panel posiada focus.\n
	 @return TRUE oznacza posiadanie focusa, FALSE jego brak.
	 */
	bool hasFocus() const;

	/** Metoda powoduje utworzenie obiektu panela z list plikw lub z podgldem
	 pliku.\n
	 @param _kindOfPanel - typ panela, patrz: @see KindOfPanel,
	 @param fileName - nazwa pliku lub katalogu jakim naley zainicjowa panel.\n
	 Rodzaj danego panela odczytywany jest z pliku konfiguracyjnego, po utworzeniu
	 panela czone s sygnay ze slotami.
	 */
	void createPanel( KindOfPanel kindOfPanel=UNKNOWNpanel, const QString & fileName=QString::null );

	/** Uruchamia kopiowanie lub przenoszenie plikw.\n
	 @param filesList - lista nazw plikw do skopiowania lub przeniesienia,
	 @param move - rwne TRUE wymusza przenoszenie, w przeciwnym razie wykonywane
	 jest kopiowanie.
	 */
	void copyFiles( const QStringList & filesList, bool move ) {
		if ( mFilesPanel ) mFilesPanel->copyFiles( filesList, move );
	}

	/** Pobiera rodzaj systemu plikw w biecym panelu.\n
	 @param kindOfFS - zmienna, w ktrej umieszczany jest rodzaj FS-u.
	 */
	void getKindOfFS( KindOfFilesSystem & kindOfFS ) {
		if ( mFilesPanel ) kindOfFS = mFilesPanel->kindOfCurrentFS();
	}

	/** Powoduje zmian rozmiaru podanej kolumny w biecym panelu.\n
	 @param section - numer kolumny, ktre naley zmieni rozmiar,
	 @param oldSize - poprzedni rozmiar podanej kolumny,
	 @param newSize - nowy rozmiar dla podanej kolumny.
	 */
	void resizeSection( int section, int oldSize, int newSize ) {
		if ( mFilesPanel ) mFilesPanel->resizeSection( section, oldSize, newSize );
	}

	/** Uruchamia odczytanie podanego katalogu.\n
	 @param pathName - nazwa katalogu od odczytania.
	 */
	void changeDirectory( const QString & pathName ) {
		if ( mFilesPanel ) mFilesPanel->slotPathChanged( pathName );
	}

	/** Ustawia skadow klasy zawierajc nazw otwartego katalogu w panelu obok.\n
	 @param url - nazwa katalogu z panelu obok.
	 */
	void setPanelBesideURL( const QString & url ) {
		if ( mFilesPanel ) mFilesPanel->setPanelBesideURL( url );
	}

	/** Metoda uruchamia uaktualnianie listy plikw.\n
	 @param updateListOp - rodzaj uaktualnienia, patrz:
	  @see ListViewExt::UpdateListOperation,
	 @param selectedItemsList - wskanik na list elementw, ktrymi naley
	 uaktualni biec,
	 @param newUrlInfo - informacja o pliku (uywana jeli uaktualniany jest jeden
	 element).
	 */
	void updateList( int updateListOp, QValueList<QListViewItem *> *selectedItemsList, const UrlInfoExt & newUrlInfo ) {
		if ( mFilesPanel ) mFilesPanel->updateList( updateListOp, selectedItemsList, newUrlInfo );
	}

	/** Metoda pobiera wskanik do listy zaznaczonych elementw w biecym panelu.\n
	 @param selectedItemsList - wskanik, pod ktry naley zapisa list elementw
	 */
	void getSelectedItemsList( QValueList<QListViewItem *> *& selectedItemsList ) {
		if ( mFilesPanel ) selectedItemsList = mFilesPanel->selectedItemsList();
	}

	/** Inicjuje baz skojarze plikw w biecym panelu podanym wskanikiem do
	 obiektu skojarze.
	 @param fa - wskanik na obiekt skojarze plikw.
	 */
	void setFilesAssociation( FilesAssociation *fa ) {
		mFilesAssociation = fa;
		if ( mFilesPanel ) mFilesPanel->setFilesAssociation( fa );
		else
		if ( mFileView )   mFileView->setFilesAssociation( fa );
	}

	/** Metoda inicjuje skrty klawiszowe w panelu.\n
	 @param keyShortcuts - wskanik na obiekt skrtw klawiszowych.
	 */
	void setKeyShortcuts( KeyShortcuts * keyShortcuts ) {
		if ( mFilesPanel ) mFilesPanel->setKeyShortcuts( keyShortcuts );
		else
		if ( mFileView )   mFileView->setKeyShortcuts( keyShortcuts );
	}

	/** Metoda uaktualnia menu zawierajce list filtrw w panelu plikw.\n
	 @param filters - wskanik na obiekt filtrw.
	 */
	void setFilters( Filters * filters ) {
		if ( mFilesPanel ) mFilesPanel->updateFiltersMenu( filters );
	}

	/** Podaje ciek lub nazw pliku w biecym panelu.\n
	 @return cieka lub nazwa pliku.
	 */
	QString currentURL() const {
		return (mFilesPanel) ? mFilesPanel->currentURL() : mFileView->currentURL();
	}

	/** Podaje wskanik do panela plikw.\n
	 @return wskanik do panela plikw.
	 */
	FilesPanel *filesPanel() const { return mFilesPanel; }

	/** Metoda uruchamia adowanie pliku do podgldu osadzonego jako panel.\n
	 @param fa - wskanik na obiekt bazy skojarze plikw,
	 @param fp - wskanik na obiekt panela plikw.\n
	 Poniewa panel nie zawiera obugi adowania, wykorzystywany jest do tego celu
	 obiekt FilesPanel-a, do ktrego jest tu przekazywany wskanik. Inicjowane s
	 tutaj te skojarzenia plikw na obiektcie panela podgldu.
	 */
	void initLoadFileToView( FilesAssociation *fa, FilesPanel *fp );


	/** Otwiera biecy plik (lub aktualnie zaznaczone) w trybie podgldu lub
	 edycji.\n
	 @param editMode - warto TRUE, wymusza tryb edycji, w przeciwnym razie
	  trybem bdzie podgld.\n
	 Metoda uywana przez klas rodzica.
	 */
	void viewFile( bool editMode ) {
		if ( mFilesPanel ) mFilesPanel->openCurrentFile( editMode );
	}

	/** Kopiuje lub przenosi biecy plik (lub aktualnie zaznaczone).\n
	 @param move - warto TRUE wymusza przenoszenie pliku, w przeciwnym razie
	  wykonywane jest kopiowanie.\n
	 Metoda uywana przez klas rodzica.
	 */
	void copyFiles( bool move ) {
		if ( mFilesPanel ) mFilesPanel->slotCopy( FALSE, move );
	}

	/** Tworzy nowy katalog.\n
	 Metoda uywana przez klas rodzica.
	 */
	void makeDir() {
		if ( mFilesPanel ) mFilesPanel->createNewEmptyFile( TRUE );
	}

	/** Usuwa biecy plik (lub aktualnie zaznaczone).\n
	 Metoda uywana przez klas rodzica.
	 */
	void deleteFile() {
		if ( mFilesPanel ) mFilesPanel->slotDelete();
	}

	/** Tworzy link(i) do wybranych elementw.
	 */
	void makeLink() {
		if ( mFilesPanel ) mFilesPanel->slotMakeLink();
	}

private:
	FilesPanel *mFilesPanel;
	FileView   *mFileView;
	QWidget    *myParent;

	FilesAssociation *mFilesAssociation;

	KindOfPanel mKindOfPanel;
	/** Kind of current view for files panel and view panel */
	int mKindOfView;
	uint mWidth;


	/** Usuwany jest tutaj obiekt panela listy plikw lub panela podgldu pliku.
	 */
	void removePanel();

protected:
	/** Obsuga zdarzenia zmiany rozmiaru dla obiektu panela.
	 */
	void resizeEvent( QResizeEvent * );

};

#endif
